----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:20:11 12/08/2013 
-- Design Name: 
-- Module Name:    GPMC_SLAVE - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;
library grlib, techmap;
use grlib.stdlib.all;
use techmap.gencomp.all;
use techmap.allclkgen.all;
library lpp;
use lpp.general_purpose.all;

entity GPMC_SLAVE is
  generic (
    memtech       : integer := 0;
    padtech       : integer := 0
  );
    Port ( 
	     clk           : in  STD_LOGIC;
        reset         : in  STD_LOGIC;
		  STATUS        : in  STD_LOGIC_VECTOR(15 downto 0);
		  DATA          : out STD_LOGIC_VECTOR(15 downto 0);
		  ADDRESS       : out std_logic_vector(19 downto 0);
		  WEN	          : out STD_LOGIC;
		  SMP_CKL       : out STD_LOGIC;
		  SMP_WEN       : out STD_LOGIC;
        GPMC_AD       : inout std_logic_vector(15 downto 0);
        GPMC_A        : in  std_logic_vector(19 downto 0);
        GPMC_CLK      : in  std_logic;
        GPMC_WEN      : in  std_logic;   
        GPMC_OEN_REN  : in  std_logic;  
        GPMC_ADVN_ALE : in  std_logic;   
        GPMC_CSN      : in  std_logic_vector(2 downto 0);
        GPMC_BE0N_CLE : in  std_logic; 
        GPMC_BE1N     : in  std_logic;
        GPMC_WAIT0    : out std_logic;
        GPMC_WPN      : in  std_logic
		  );
end GPMC_SLAVE;

architecture Behavioral of GPMC_SLAVE is

signal data_out     : std_logic_vector(15 downto 0) := (others => '0');
signal data_in      : std_logic_vector(15 downto 0) := (others => '0');
signal data_in_reg0  : std_logic_vector(15 downto 0) := (others => '0');
signal data_in_reg1  : std_logic_vector(15 downto 0) := (others => '0');
signal data_in_reg2  : std_logic_vector(15 downto 0) := (others => '0');
signal address_reg0  : std_logic_vector(19 downto 0) := (others => '0');
signal address_reg1  : std_logic_vector(19 downto 0) := (others => '0');
signal address_reg2  : std_logic_vector(19 downto 0) := (others => '0');
signal ADVN_ALE_reg  : std_logic_vector(3 downto 0)  := (others => '0');


signal GPMC_CLK_reg : std_logic_vector(3 downto 0)  := (others => '0');
signal data_r       : std_logic_vector(15 downto 0) := (others => '0');
signal GPMC_WEN_reg : std_logic_vector(3 downto 0)  := (others => '0');

signal outen       : std_logic := '0';


begin

outen    <= GPMC_OEN_REN or GPMC_CSN(0);
data_out <= STATUS;

data_pad : iopadv generic map (tech=> padtech,width => 16)
port map (
   pad => GPMC_AD(15 downto 0),
   o   => data_in(15 downto 0),
   en  => outen,
   i   => data_out(15 downto 0)
);

GPMC_WAIT0  <=  '1';
SMP_CKL     <=  GPMC_CLK_reg(0);
SMP_WEN     <= GPMC_WEN_reg(2);

process(reset,clk)
begin
if reset = '0' then
	GPMC_CLK_reg <= "0000";
	ADDRESS		 <=  (others => '0');
	ADVN_ALE_reg <=  (others => '0');
	address_reg0 <=  (others => '0');
	address_reg1 <=  (others => '0');
	address_reg2 <=  (others => '0');
elsif clk'event and clk = '1' then
	GPMC_CLK_reg(0) <= GPMC_CLK;
	GPMC_CLK_reg(1) <= GPMC_CLK_reg(0);
	GPMC_CLK_reg(2) <= GPMC_CLK_reg(1);
	ADVN_ALE_reg(0) <= GPMC_ADVN_ALE;
	ADVN_ALE_reg(1) <= ADVN_ALE_reg(0);
   ADVN_ALE_reg(2) <= ADVN_ALE_reg(1);	
	address_reg0    <= GPMC_A;
	address_reg1    <= address_reg0;
	address_reg2    <= address_reg1;
	if GPMC_CLK_reg(1) = '1' and GPMC_CLK_reg(2) = '0' then
	    if ADVN_ALE_reg(2) = '0' then
          ADDRESS  <= address_reg2;
	    end if;		
	end if;
	
end if;
end process;


process(reset,clk)
begin
if reset = '0' then
	WEN	<= '1';
	GPMC_WEN_reg <= "0000";
	data_in_reg0 <=  (others => '0');
	data_in_reg1 <=  (others => '0');
	data_in_reg2 <=  (others => '0');
elsif clk'event and clk = '1' then
	GPMC_WEN_reg(0)    <= GPMC_WEN;
	GPMC_WEN_reg(1)    <= GPMC_WEN_reg(0);
	GPMC_WEN_reg(2)    <= GPMC_WEN_reg(1);
	data_in_reg0       <= data_in;
	data_in_reg1       <= data_in_reg0;
	data_in_reg2       <= data_in_reg1;
	if GPMC_WEN_reg(2) = '1' and GPMC_WEN_reg(1) = '0' then
		WEN   <= '0';
		DATA  <= data_in_reg2;
	else
		WEN   <= '1';
	end if;
end if;
end process;

end Behavioral;














